#include "X3D_ColorInterpolator.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

ColorInterpolator::ColorInterpolator()
  : X3DInterpolatorNode(),
    _keyValue(MFColor())
{
  defineTypeName("ColorInterpolator", "Interpolation");
}

ColorInterpolator::ColorInterpolator(const MFFloat &key,
                                     const MFColor &keyValue)
  : X3DInterpolatorNode(key),
    _keyValue(keyValue)
{
  defineTypeName("ColorInterpolator", "Interpolation");
}

ColorInterpolator::ColorInterpolator(const ColorInterpolator &N)
  : X3DInterpolatorNode(N),
    _keyValue(N._keyValue)
{
}

SFAbstractNode ColorInterpolator::clone() const
{
  return new ColorInterpolator(*this);
}

ColorInterpolator::~ColorInterpolator()
{
}

void ColorInterpolator::setKeyValue(const MFColor &keyValue)
{
  _keyValue = keyValue;
}

void ColorInterpolator::load(const X3DFileElement *element)
{
  X3DInterpolatorNode::load(element);

  int index;
  index = element->getIndexAttribute("keyValue");
  if (index != -1)
    _keyValue = MFColor(element->getAttribute(index));
}

SFString &ColorInterpolator::write(SFString &output) const
{
  X3DInterpolatorNode::write(output);
  
  if (!_keyValue.empty())
    output += " keyValue=\"" + toSFString(_keyValue) + "\"";

  return output;  
}

}
}
