#include "X3D_Collision.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

Collision::Collision()
: X3DGroupingNode(), X3DBoundedObject()/*, X3DSensorNode()*/, _collide(true), _proxy(0)
{
  defineTypeName("Collision", "Navigation");
}

Collision::Collision(/*const SFBool &enabled, */const SFBool &collide, const SFVec3f &bboxCenter, const SFVec3f &bboxSize, X3DChildNode *proxy)
: X3DGroupingNode(), X3DBoundedObject(bboxCenter, bboxSize)/*, X3DSensorNode(enabled)*/, _collide(collide)
{
  defineTypeName("Collision", "Navigation");
  
  setProxy(static_cast<X3DChildNode *>(proxy));  
}
          
Collision::Collision(const Collision &C)
: X3DGroupingNode(C), X3DBoundedObject(C)/*, X3DSensorNode(C)*/, _collide(true), _proxy(0)
{
  setProxy(static_cast<X3DChildNode *>(C._proxy));
}

SFAbstractNode Collision::clone() const
{
  return new Collision(*this);
}

Collision::~Collision()
{
  removeParentFromChild(this, _proxy);
}

void Collision::setCollide(const SFBool &collide)
{
  _collide = collide;
}

void Collision::setProxy(X3DChildNode *proxy)
{
  removeParentFromChild(this, _proxy);
  _proxy = proxy;
  addParentToChild(this, _proxy);
}

MFAbstractNode Collision::getChildList() const
{
  MFAbstractNode childrenList;
  if (_proxy != 0)
    childrenList.push_back(_proxy);
    
  return childrenList;  
}

void Collision::load(const X3DFileElement *element)
{
//  X3DSensorNode::load(element);

  int index;
  index = element->getIndexAttribute("collide");
  if (index != -1)
    _collide = (element->getAttribute(index)).upper() == "TRUE";
}

SFString &Collision::write(SFString &output) const
{
  //  X3DSensorNode::write(output);

  if (!_collide)
    output += " collide=\"" + toSFString(_collide) + "\""; 

  return output;
}

}
}

