#include "MESH_ConnectedComComputerCoreVisitor.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

ConnectedComComputerCoreVisitor::ConnectedComComputerCoreVisitor(Progress *progress)
: CoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&ConnectedComComputerCoreVisitor::enterVertex);
  defineWalkOnFunction(&ConnectedComComputerCoreVisitor::walkOnShape);
  
  ConnectedComComputerStateVariables *stateVariables = GraphTraversal::getInstanceOf<ConnectedComComputerStateVariables>();  
  stateVariables->setProgress(progress);
}

bool ConnectedComComputerCoreVisitor::walkOnShape(Shape *, SFAbstractNode child) 
{
  ConnectedComComputerStateVariables *stateVariables = GraphTraversal::getInstanceOf<ConnectedComComputerStateVariables>();
  MFNode selectedNodes = stateVariables->getSelectedNodes();
  return (selectedNodes.empty() || (std::find(selectedNodes.begin(), selectedNodes.end(), child) != selectedNodes.end()));
}

void ConnectedComComputerCoreVisitor::enterVertex(DefVertex *V) 
{
  ConnectedComComputerStateVariables *stateVariables = GraphTraversal::getInstanceOf<ConnectedComComputerStateVariables>();

  if (!stateVariables->getNode(V))
  {
    // computing the list of Mesh parents of V.
    MFAbstractNode parentList = V->getParentList();
    std::list<DefMesh *> meshList;
    for (MFAbstractNode::const_iterator it = parentList.begin(); it != parentList.end(); ++it)
    {
      if (dynamic_cast<DefMesh *>(*it) != 0)
        meshList.push_back(static_cast<DefMesh *>(*it));
    }
    
    // defining structures.
    std::vector<DefSFFace *> stack;
    unsigned int stackSize = 0;
    for (std::list<DefMesh *>::const_iterator it = meshList.begin(); it != meshList.end(); ++it)
      stackSize += (*it)->getFaces().size();
    stack.reserve(stackSize);
    stateVariables->getProgress()->setTotalSteps(stackSize);
    unsigned int step = 0;
    
    std::list<DefMesh *>::const_iterator itMesh = meshList.begin();
    if (itMesh == meshList.end()) return;
    
    DefMesh::MFFace::const_iterator itFace = (*itMesh)->getFaces().begin();
    
    while (itMesh != meshList.end())
    {
      std::list<DefMesh *> cmeshList;
      std::map<DefMesh::SFFace *, DefMesh *> faceMap;
      stack.push_back(*itFace);
      stateVariables->getProgress()->setProgress(++step);

      std::vector<DefSFFace *>::const_iterator f = stack.end();
      --f;
      faceMap[*f] = *itMesh;
      if (std::find(cmeshList.begin(), cmeshList.end(), *itMesh) == cmeshList.end())
        cmeshList.push_back(*itMesh);
        
      while (f != stack.end())
      {
        const DefSFFace::MFEdge &me = (*f)->getEdges();
        for (DefSFFace::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
        { 
          const DefSFEdge::MFFace &mlf = (*e)->getLeftFaces();
          const DefSFEdge::MFFace &mrf = (*e)->getRightFaces();
        
          for (DefSFEdge::MFFace::const_iterator nf = mlf.begin(); nf != mlf.end(); ++nf)
          {
            if (faceMap.find(*nf) == faceMap.end())
            {
              // finding the Mesh                            
              for (std::list<DefMesh *>::const_iterator it = meshList.begin(); it != meshList.end(); ++it)
              {
                const DefMesh::MFFace &faces = (*it)->getFaces();
                if (faces.find(*nf) != faces.end())
                {
                  faceMap[*nf] = *it;
                    
                  if (std::find(cmeshList.begin(), cmeshList.end(), *it) == cmeshList.end())
                    cmeshList.push_back(*it);

                  break;    
                }
              }  
              
              stack.push_back(*nf);
              stateVariables->getProgress()->setProgress(++step);
            }
          }
        
          for (DefSFEdge::MFFace::const_iterator nf = mrf.begin(); nf != mrf.end(); ++nf)
          {
            if (faceMap.find(*nf) == faceMap.end())
            {
              // finding the Mesh                            
              for (std::list<DefMesh *>::const_iterator it = meshList.begin(); it != meshList.end(); ++it)
              {
                const DefMesh::MFFace &faces = (*it)->getFaces();
                if (faces.find(*nf) != faces.end())
                {
                  faceMap[*nf] = *it;
                    
                  if (std::find(cmeshList.begin(), cmeshList.end(), *it) == cmeshList.end())
                    cmeshList.push_back(*it);

                  break;    
                }
              }  

              stack.push_back(*nf);
              stateVariables->getProgress()->setProgress(++step);
            }
          }
        }
        ++f;
      }
      
      // creating new Meshs and Vertex.
      std::map<DefMesh *, DefMesh *> meshMap;
      DefVertex *newVertexNode = new DefVertex();
      for (std::list<DefMesh *>::const_iterator it = cmeshList.begin(); it != cmeshList.end(); ++it)
      {
        DefMesh *newMesh = new DefMesh();
        newMesh->setVertex(newVertexNode);
        
        // filling Mesh datas.
        newMesh->data().setNormal((*it)->data().hasNormal());
        newMesh->data().setNormalPerVertex((*it)->data().getNormalPerVertex());

        newMesh->data().setColor((*it)->data().hasColor());
        newMesh->data().setColorPerVertex((*it)->data().getColorPerVertex());
        newMesh->data().setRGBA((*it)->data().getRGBA());

        newMesh->data().setTexCoord((*it)->data().hasTexCoord());

        meshMap[*it] = newMesh;
      }
      
      // iterating the faces of the connected component.
      std::map<unsigned int, unsigned int> newIndex;
      for (std::map<DefMesh::SFFace *, DefMesh *>::const_iterator it = faceMap.begin(); it != faceMap.end(); ++it)
      {
        MFInt32 faceCoordIndex;
        DefSFFace *face = (*it).first;
        const DefSFFace::MFEdge &me = face->getEdges();
        for (DefSFFace::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
        {
          DefSFVertex *v = (*e)->getFromVertex();
          if (newIndex.find(v->getIndex()) == newIndex.end())
          {
            DefSFVertex *newVertex = newVertexNode->createVertex();
            
            newVertex->data().setPoint(v->data().getPoint());
            
            newIndex[v->getIndex()] = newVertex->getIndex();
          }
          faceCoordIndex.push_back(newIndex[v->getIndex()]);
        }
        
        // filling face data.
        DefSFFace *newFace = meshMap[(*it).second]->createFace(faceCoordIndex);

        newFace->data().setNormal(face->data().getNormal());
        newFace->data().setColor(face->data().getColor());

        // filling vertex data.
        const DefSFFace::MFEdge &nme = newFace->getEdges();
        DefSFFace::MFEdge::const_iterator e = me.begin();
        DefSFFace::MFEdge::const_iterator ne = nme.begin();
        while (e != me.end())
        {
          DefSFVertex *v = (*e)->getFromVertex();
          DefSFVertex *newVertex = (*ne)->getFromVertex();
          
          newVertex->data().setNormalOfFace(v->data().getNormalOfFace(face), newFace);
          newVertex->data().setColorOfFace(v->data().getColorOfFace(face), newFace); 
          newVertex->data().setTexCoordOfFace(v->data().getTexCoordOfFace(face), newFace);  
          
          ++e;
          ++ne;  
        }
      }
      
      // adding new Meshes to the scene graph.
      for (std::map<DefMesh *, DefMesh *>::const_iterator it = meshMap.begin(); it != meshMap.end(); ++it)
      {
        DefMesh *mesh = (*it).first;
        DefMesh *newMesh = (*it).second;
        
        MFAbstractNode parentList = mesh->getParentList();
        for (MFAbstractNode::const_iterator itParent = parentList.begin(); itParent != parentList.end(); ++itParent)
        {
          SFAbstractNode cparent = (*itParent)->clone();
          cparent->setChild(newMesh);
          
          MFAbstractNode pparentList = (*itParent)->getParentList();
          for (MFAbstractNode::const_iterator itPParent = pparentList.begin(); itPParent != pparentList.end(); ++itPParent)
            (*itPParent)->addChild(cparent);
        }
      }

      // finding the next face.
      while ((itMesh != meshList.end()) && (std::find(stack.begin(), stack.end(), *itFace) != stack.end()))
      {
        ++itFace;
        if (itFace == (*itMesh)->getFaces().end())
        {
          ++itMesh;
          if (itMesh != meshList.end())
            itFace = (*itMesh)->getFaces().begin();
        }    
      }
    }

    // remove old meshes.
    for (std::list<DefMesh *>::const_iterator it = meshList.begin(); it != meshList.end(); ++it)
    {
      MFAbstractNode parentList = (*it)->getParentList();
      for (MFAbstractNode::const_iterator itParent = parentList.begin(); itParent != parentList.end(); ++itParent)
      {
        MFAbstractNode pparentList = (*itParent)->getParentList();
        for (MFAbstractNode::const_iterator itPParent = pparentList.begin(); itPParent != pparentList.end(); ++itPParent)
        {
          (*itPParent)->removeChild(*itParent);
          stateVariables->addToDeleteNode(*itParent);
        }
      }
    }
    stateVariables->addNode(V);
  }
}

}
}  
